@@ -6,42 +6,56 @@
 __author__ = """Eric BREHAULT <eric.brehault@makina-corpus.org>"""
 __docformat__ = 'plaintext'
 
-from Products.Archetypes.config import RENAME_AFTER_CREATION_ATTEMPTS
+# From the standard library
+from copy import deepcopy
+from time import strptime
+
+# 3rd party Python 
+from jsonutil import jsonutil as json
+
+# Zope
 from AccessControl import ClassSecurityInfo
-from zope.interface import implements
-import interfaces
-from Products.CMFCore.utils import getToolByName
-from Products.CMFPlone.utils import normalizeString
-from OFS.ObjectManager import BadRequestException
-
-from exceptions import PlominoScriptException
-from Products.CMFPlomino.config import *
-
-import transaction
-from interfaces import *
 from AccessControl import Unauthorized
+from DateTime import DateTime
+from interfaces import *
+from persistent.dict import PersistentDict
+from zope.annotation import IAttributeAnnotatable
+# 4.3 compatibility
+try:
+    from zope.container.contained import Contained
+except ImportError:
+    from zope.app.container.contained import Contained
+from zope.component.factory import Factory
+from zope.component import queryUtility
+from zope import event
+from zope.interface import implements
+from zope.interface import Interface
+import transaction
+
 try:
     from AccessControl.class_init import InitializeClass
 except:
     from App.class_init import InitializeClass
-from time import strptime
-from DateTime import DateTime
-from zope import event
-from zope.component import queryUtility
-from zope.interface import implements, Interface
-from zope.component.factory import Factory
-from Products.CMFCore.CMFBTreeFolder import CMFBTreeFolder
+
+# CMF/Plone
+from OFS.ObjectManager import BadRequestException
+from Products.Archetypes.config import RENAME_AFTER_CREATION_ATTEMPTS
 from Products.BTreeFolder2.BTreeFolder2 import BTreeFolder2Base
+from Products.CMFCore.CMFBTreeFolder import CMFBTreeFolder
+from Products.CMFCore.exceptions import BadRequest
+from Products.CMFCore.utils import getToolByName
+from Products.CMFPlone.utils import normalizeString
+
 try:
     from Products.CMFCore.CMFCatalogAware import CatalogAware
 except:
     from Products.CMFCore.CMFCatalogAware import CMFCatalogAware as CatalogAware
-from zope.annotation import IAttributeAnnotatable
-from zope.app.container.contained import Contained
 
-import simplejson as json
-from copy import deepcopy
-from persistent.dict import PersistentDict
+# Plomino
+from exceptions import PlominoScriptException
+from Products.CMFPlomino.config import *
+import interfaces
+
 
 import logging
 logger = logging.getLogger('Plomino')
@@ -53,7 +67,7 @@ try:
     URL_NORMALIZER = True
 except ImportError:
     URL_NORMALIZER = False
-    
+
 from PlominoUtils import DateToString, StringToDate, sendMail, asUnicode, asList, PlominoTranslate
 from OFS.Image import File
 from ZPublisher.HTTPRequest import FileUpload
@@ -69,17 +83,21 @@ except ImportError, e:
     HAS_BLOB = False
 
 class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
+    """ These represent the contents in a Plomino database.
+
+    A document contains *items* that may or may not correspond to fields on
+    one or more forms.
     """
-    """
+
     security = ClassSecurityInfo()
     implements(interfaces.IPlominoDocument, IAttributeAnnotatable)
-    
+
     portal_type = "PlominoDocument"
     meta_type = "PlominoDocument"
-    
+
     security.declarePublic('__init__')
     def __init__(self, id):
-        """initialization
+        """ Initialization
         """
         CMFBTreeFolder.__init__(self, id)
         self.id = id
@@ -89,6 +107,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
     security.declarePublic('checkBeforeOpenDocument')
     def checkBeforeOpenDocument(self):
         """ Check read permission and open view.
+
         .. NOTE:: if ``READ_PERMISSION`` is set on the ``view`` action
             itself, it causes an error ('maximum recursion depth exceeded')
             if user hasn't permission.
@@ -97,7 +116,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
             return self.OpenDocument()
         else:
             raise Unauthorized, "You cannot read this content"
-        
+
     def doc_path(self):
         return self.getPhysicalPath()
 
@@ -112,7 +131,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
             return self.REQUEST.physicalPathToURL(short_path)
         else:
             return "/".join(short_path)
-    
+
     security.declarePublic('setItem')
     def setItem(self,name,value):
         """
@@ -175,7 +194,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
 
     security.declarePublic('getRenderedItem')
     def getRenderedItem(self, itemname, form=None, formid=None, convertattachments=False):
-        """ Return the item rendered according to the corresponding field. 
+        """ Return the item rendered according to the corresponding field.
 
         The used form can be, in order of precedence:
         - passed as the `form` parameter,
@@ -185,7 +204,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
         If no form or field is found, return the empty string.
 
         If `convertattachments` is True, then we assume that field
-        attachments are text and append them to the rendered value. 
+        attachments are text and append them to the rendered value.
         """
         db = self.getParentDatabase()
         result = ''
@@ -198,7 +217,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
             field = form.getFormField(itemname)
             if field:
                 result = field.getFieldRender(form, self, False)
-                if (field.getFieldType() == 'ATTACHMENT' and 
+                if (field.getFieldType() == 'ATTACHMENT' and
                     convertattachments):
                     result += ' ' + db.getIndex().convertFileToText(self, itemname).decode('utf-8')
                     result = result.encode('utf-8')
@@ -206,20 +225,27 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
         return result
 
     security.declarePublic('tojson')
-    def tojson(self, REQUEST=None, item=None, formid=None):
+    def tojson(self, REQUEST=None, item=None, formid=None, rendered=False):
         """return item value as JSON
         (return all items if item=None)
         """
         if not self.isReader():
             raise Unauthorized, "You cannot read this content"
-        
+
+        datatables_format = False
         if REQUEST:
             REQUEST.RESPONSE.setHeader('content-type', 'application/json; charset=utf-8')
             item = REQUEST.get('item', item)
             formid = REQUEST.get('formid', formid)
+            rendered_str = REQUEST.get('rendered', None)
+            if rendered_str:
+                rendered = True
+            datatables_format_str = REQUEST.get('datatables', None)
+            if datatables_format_str:
+                datatables_format = True
         if not item:
             return json.dumps(self.items.data)
-        
+
         if not formid:
             form = self.getForm()
         else:
@@ -227,13 +253,24 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
         if form:
             field = form.getFormField(item)
             if field:
-                adapt = field.getSettings()
-                fieldvalue = adapt.getFieldValue(form, self, False, False, REQUEST)
+                if field.getFieldType() == 'DATAGRID':
+                    adapt = field.getSettings()
+                    fieldvalue = adapt.getFieldValue(form, self, False, False, REQUEST)
+                    fieldvalue = adapt.rows(fieldvalue, rendered=rendered)
+                    if datatables_format:
+                        fieldvalue = { 'iTotalRecords': len(fieldvalue), 'aaData': fieldvalue }
+                else:
+                    if rendered:
+                        fieldvalue = self.getRenderedItem(item, form)
+                    else:
+                        adapt = field.getSettings()
+                        fieldvalue = adapt.getFieldValue(form, self, False, False, REQUEST)
             else:
                 fieldvalue = self.getItem(item)
         else:
             fieldvalue = self.getItem(item)
-        return json.dumps(fieldvalue) 
+
+        return json.dumps(fieldvalue)
 
     security.declarePublic('computeItem')
     def computeItem(self, itemname, form=None, formid=None, store=True, report=True):
@@ -268,7 +305,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
         """
         """
         return self.getParentDatabase().isCurrentUserReader(self)
-        
+
     security.declarePublic('isAuthor')
     def isAuthor(self):
         """
@@ -297,7 +334,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
             return self.errors_json(errors=json.dumps({'success': False,'errors':errors}))
         else:
             return self.errors_json(errors=json.dumps({'success': True}))
-        
+
     security.declareProtected(EDIT_PERMISSION, 'saveDocument')
     def saveDocument(self, REQUEST, creation=False):
         """save a document using the form submitted content
@@ -328,7 +365,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
     security.declareProtected(EDIT_PERMISSION, 'refresh')
     def refresh(self, form=None):
         """ re-compute fields and re-index document
-        (onSave event is not called, and authors are not updated 
+        (onSave event is not called, and authors are not updated
         """
         self.save(form, creation=False, refresh_index=True, asAuthor=False, onSaveEvent=False)
 
@@ -355,14 +392,23 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
                     pass
 
             # compute the document title
-            try:
-                result = self.runFormulaScript("form_"+form.id+"_title", self, form.DocumentTitle)
-            except PlominoScriptException, e:
-                e.reportError('Title formula failed')
-                result = ""
-            if not result:
-                result = form.Title()
-            self.setTitle(result)
+            title_formula = form.getDocumentTitle()
+            if title_formula:
+                # Use the formula if we have one
+                try:
+                    title = self.runFormulaScript("form_"+form.id+"_title", self, form.DocumentTitle)
+                    if title != self.Title():
+                        self.setTitle(title)
+                except PlominoScriptException, e:
+                    e.reportError('Title formula failed')
+            elif creation:
+                # If we have no formula and we're creating, use Form's title
+                title = form.Title()
+                if title != self.Title():
+                    # We may be calling save with 'creation=True' on
+                    # existing documents, in which case we may already have
+                    # a title.
+                    self.setTitle(title)
 
             # update the document id
             if creation and form.getDocumentId():
@@ -370,7 +416,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
                 if new_id:
                     transaction.savepoint(optimistic=True)
                     db.documents.manage_renameObject(self.id, new_id)
-            
+
         # update the Plomino_Authors field with the current user name
         if asAuthor:
             authors = self.getItem('Plomino_Authors')
@@ -403,6 +449,24 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
             if db.getIndexInPortal():
                 db.portal_catalog.catalog_object(self, "/".join(db.getPhysicalPath() + (self.id,)))
 
+    def _onOpenDocument(self, form=None):
+        """ execute the onOpenDocument code of the form
+        """
+        if not form:
+            form = self.getForm()
+
+        onOpenDocument_error = ''
+        try:
+            if form.getOnOpenDocument():
+                onOpenDocument_error = self.runFormulaScript(
+                        "form_"+form.id+"_onopen",
+                        self,
+                        form.onOpenDocument)
+                return onOpenDocument_error
+        except PlominoScriptException, e:
+            e.reportError('onOpenDocument event failed')
+
+
     security.declareProtected(READ_PERMISSION, 'openWithForm')
     def openWithForm(self, form, editmode=False):
         """ Display the document using the given form's layouts.
@@ -417,16 +481,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
             if not self.isReader():
                 raise Unauthorized, "You cannot read this content"
 
-        # execute the onOpenDocument code of the form
-        onOpenDocument_error = ''
-        try:
-            if form.getOnOpenDocument():
-                onOpenDocument_error = self.runFormulaScript(
-                        "form_"+form.id+"_onopen",
-                        self,
-                        form.onOpenDocument)
-        except PlominoScriptException, e:
-            e.reportError('onOpenDocument event failed')
+        onOpenDocument_error = self._onOpenDocument(form)
 
         if onOpenDocument_error:
             html_content = onOpenDocument_error
@@ -448,7 +503,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
 
         plone_tools = getToolByName(db, 'plone_utils')
         encoding = plone_tools.getSiteEncoding()
-        html_content = html_content.encode(encoding) 
+        html_content = html_content.encode(encoding)
 
         return html_content
 
@@ -504,7 +559,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
                     return PortalContent.__getattr__(self, name)
                 except Exception, e:
                     raise AttributeError, name
-            else:   
+            else:
                 raise AttributeError, name
 
     security.declareProtected(READ_PERMISSION, 'isSelectedInView')
@@ -570,7 +625,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
 
         for itemname in self.items.keys():
             item_value = self.getItem(itemname)
-            if type(item_value) is list: 
+            if type(item_value) is list:
                 for v in item_value:
                     if type(v) is list:
                         values = values + [asUnicode(k) for k in v]
@@ -591,12 +646,16 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
         return ' '.join(values)
 
     security.declareProtected(READ_PERMISSION, 'getfile')
-    def getfile(self, filename=None, REQUEST=None):
+    def getfile(self, filename=None, REQUEST=None, asFile=False):
         """
         """
         if not self.isReader():
             raise Unauthorized, "You cannot read this content"
 
+        onOpenDocument_error = self._onOpenDocument()
+        if onOpenDocument_error:
+            raise Unauthorized, onOpenDocument_error
+
         fss = self.getParentDatabase().getStorageAttachments()
         if REQUEST is not None:
             filename = REQUEST.get('filename')
@@ -609,15 +668,15 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
                 file_obj = self.get(filename, None)
             if not file_obj:
                 return None
-            if REQUEST is None:
+            if asFile:
                 return file_obj
-            else:
+            if REQUEST:
                 REQUEST.RESPONSE.setHeader('content-type', file_obj.getContentType())
                 REQUEST.RESPONSE.setHeader("Content-Disposition", "inline; filename="+filename)
-                if fss:
-                    return file_obj.getData()
-                else:
-                    return file_obj.data
+            if fss:
+                return file_obj.getData()
+            else:
+                return file_obj.data
         else:
             return None
 
@@ -645,11 +704,14 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
             if """\\""" in filename:
                 filename=filename.split("\\")[-1]
             filename = ".".join([normalizeString(s, encoding='utf-8') for s in filename.split('.')])
-            if filename in self.objectIds():
-                if overwrite:
-                    self.deletefile(filename)
-                else:
-                    return ("ERROR: "+filename+" already exists", "")
+            if overwrite and filename in self.objectIds():
+                self.deletefile(filename)
+            try:
+                self._checkId(filename)
+            except BadRequest:
+                # if filename is a reserved id, we rename it
+                filename = DateTime().toZone('UTC').strftime("%Y%m%d%H%M%S") + "_" + filename
+            
             if(self.getParentDatabase().getStorageAttachments()==True):
                 tmpfile=File(filename, filename, submittedValue)
                 storage = FileSystemStorage();
@@ -780,7 +842,7 @@ class PlominoDocument(CatalogAware, CMFBTreeFolder, Contained):
             new_id = IUserPreferredURLNormalizer(request).normalize(result)
         else:
             new_id = queryUtility(IURLNormalizer).normalize(result)
-        
+
         # check if the id already exists
         documents = self.getParentDatabase().documents
         try:
@@ -820,7 +882,7 @@ class TemporaryDocument(PlominoDocument):
         """ Temporary docs are able to acquire from the db
         """
         return self._parent
-        
+
     security.declarePublic('isEditMode')
     def isEditMode(self):
         """
@@ -842,7 +904,7 @@ class TemporaryDocument(PlominoDocument):
         """
         """
         return self._REQUEST
-    
+
     security.declarePublic('id')
     @property
     def id(self):
