@@ -27,7 +27,7 @@ from Products.CMFPlomino.exceptions import PlominoDesignException
 
 import sys
 import re
-import simplejson as json
+from jsonutil import jsonutil as json
 
 from Products.CMFCore.utils import getToolByName
 
@@ -254,7 +254,7 @@ class PlominoForm(ATFolder):
 
     security.declareProtected(CREATE_PERMISSION, 'createDocument')
     def createDocument(self,REQUEST):
-        """ Create a document using the form's submitted content.
+        """create a document using the forms submitted content
         """
         db = self.getParentDatabase()
 
@@ -280,7 +280,7 @@ class PlominoForm(ATFolder):
             tmp.setItem("Plomino_Parent_Field", parent_field)
             tmp.setItem("Plomino_Parent_Form", parent_form)
             tmp.setItem(parent_field+"_itemnames", [
-                f.getId() for f in self.getFormFields() 
+                f.getId() for f in self.getFormFields()
                 if not f.getFieldMode() == 'DISPLAY'])
             return self.ChildForm(temp_doc=tmp)
 
@@ -295,7 +295,7 @@ class PlominoForm(ATFolder):
             e.reportError('Document is created, but onCreate formula failed')
 
         if valid is None or valid=='':
-            doc.saveDocument(REQUEST, True)
+            doc.saveDocument(REQUEST, creation=True)
         else:
             db.documents._delOb(doc.id)
             db.writeMessageOnPage(valid, REQUEST, False)
@@ -305,11 +305,10 @@ class PlominoForm(ATFolder):
     def getFormFields(self, includesubforms=False, doc=None, applyhidewhen=False):
         """get fields
         """
-#        fieldlist = self.portal_catalog.search({'portal_type' : ['PlominoField'], 'path': '/'.join(self.getPhysicalPath())})
-#        result = [f.getObject() for f in fieldlist]
         fieldlist = self.objectValues(spec='PlominoField')
         result = [f for f in fieldlist]
         if applyhidewhen:
+            doc = doc or TemporaryDocument(self.getParentDatabase(), self, self.REQUEST)
             layout = self.applyHideWhen(doc)
             result = [f for f in result if """<span class="plominoFieldClass">%s</span>""" % f.id in layout]
         result.sort(key=lambda elt: elt.id.lower())
@@ -328,7 +327,6 @@ class PlominoForm(ATFolder):
     def getHidewhenFormulas(self):
         """Get hidden formulae
         """
-        #list = self.portal_catalog.search({'portal_type' : ['PlominoHidewhen'], 'path': '/'.join(self.getPhysicalPath())})
         hidewhens = self.objectValues(spec='PlominoHidewhen')
         return [h for h in hidewhens]
 
@@ -336,15 +334,12 @@ class PlominoForm(ATFolder):
     def getActions(self, target, hide=True, parent_id=None):
         """Get actions
         """
-        #all = self.portal_catalog.search({'portal_type' : ['PlominoAction'], 'path': '/'.join(self.getPhysicalPath())})
         all = self.objectValues(spec='PlominoAction')
 
         filtered = []
         for obj_a in all:
-            #obj_a=a.getObject()
             if hide:
                 try:
-                    #result = RunFormula(target, obj_a.getHidewhen())
                     result = self.runFormulaScript("action_"+self.id+"_"+obj_a.id+"_hidewhen", target, obj_a.Hidewhen, True, parent_id)
                 except PlominoScriptException, e:
                     e.reportError('"%s" hide-when formula failed' % obj_a.Title())
@@ -362,7 +357,7 @@ class PlominoForm(ATFolder):
         """
         cacheformulas = self.objectValues(spec='PlominoCache')
         return [c for c in cacheformulas]
-    
+
     security.declarePublic('getFormName')
     def getFormName(self):
         """ Return the form name
@@ -442,7 +437,7 @@ class PlominoForm(ATFolder):
             else:
                 action_render=''
             html_content = html_content.replace('<span class="plominoActionClass">'+actionName+'</span>', action_render)
-        
+
         # store fragment to cache
         html_content = self.updateCache(html_content, to_be_cached)
         return html_content
@@ -472,14 +467,18 @@ class PlominoForm(ATFolder):
 
         return html
 
+    security.declarePrivate('_get_html_content')
+    def _get_html_content(self):
+        plone_tools = getToolByName(self, 'plone_utils')
+        encoding = plone_tools.getSiteEncoding()
+        html_content = self.getField('FormLayout').getRaw(self).decode(encoding)
+        return html_content.replace('\n', '')
+
     security.declareProtected(READ_PERMISSION, 'applyHideWhen')
     def applyHideWhen(self, doc=None, silent_error=True):
         """evaluate hide-when formula and return resulting layout
         """
-        plone_tools = getToolByName(self, 'plone_utils')
-        encoding = plone_tools.getSiteEncoding()
-        html_content = self.getField('FormLayout').getRaw(self).decode(encoding)
-        html_content = html_content.replace('\n', '')
+        html_content = self._get_html_content()
 
         # remove the hidden content
         for hidewhen in self.getHidewhenFormulas():
@@ -525,14 +524,18 @@ class PlominoForm(ATFolder):
         for hidewhen in self.getHidewhenFormulas():
            if getattr(hidewhen, 'isDynamicHidewhen', False):
                return True
+        for subformname in self.getSubforms():
+            form = self.getParentDatabase().getForm(subformname)
+            if form.hasDynamicHidewhen():
+                return True
         return False
 
     security.declareProtected(READ_PERMISSION, 'getHidewhenAsJSON')
-    def getHidewhenAsJSON(self, REQUEST):
+    def getHidewhenAsJSON(self, REQUEST, parent_form=None):
         """Return a JSON object to dynamically show or hide hidewhens (works only with isDynamicHidewhen)
         """
         result = {}
-        target = TemporaryDocument(self.getParentDatabase(), self, REQUEST)
+        target = TemporaryDocument(self.getParentDatabase(), parent_form or self, REQUEST)
         for hidewhen in self.getHidewhenFormulas():
             if getattr(hidewhen, 'isDynamicHidewhen', False):
                 try:
@@ -541,7 +544,11 @@ class PlominoForm(ATFolder):
                     e.reportError('%s hide-when formula failed' % hidewhen.id)
                     #if error, we hide anyway
                     isHidden = True
-                result[hidewhen.id] = isHidden 
+                result[hidewhen.id] = isHidden
+        for subformname in self.getSubforms():
+            form = self.getParentDatabase().getForm(subformname)
+            form_hidewhens = json.loads(form.getHidewhenAsJSON(REQUEST, parent_form=parent_form or self))
+            result.update(form_hidewhens)
 
         return json.dumps(result)
 
@@ -625,8 +632,12 @@ class PlominoForm(ATFolder):
             except PlominoScriptException, e:
                 e.reportError('beforeCreate formula failed')
 
+        tmp = None
+        if hasattr(self, 'REQUEST'):
+            # hideWhens need a TemporaryDocument
+            tmp = TemporaryDocument(self.getParentDatabase(), self, self.REQUEST)
         if (not invalid) or self.hasDesignPermission(self):
-            return self.displayDocument(None, editmode=True, creation=True, request=request)
+            return self.displayDocument(tmp, editmode=True, creation=True, request=request)
         else:
             self.REQUEST.RESPONSE.redirect(self.getParentDatabase().absolute_url()+"/ErrorMessages?disable_border=1&error="+invalid)
 
@@ -675,9 +686,18 @@ class PlominoForm(ATFolder):
         """return true if the form contains at least one DateTime field
         or a datagrid (as a datagrid may contain a date)
         """
-        fields = self.getFormFields(includesubforms=True, applyhidewhen=True)
+        return self._has_fieldtypes(["DATETIME", "DATAGRID"])
+
+    security.declarePrivate('_has_fieldtypes')
+    def _has_fieldtypes(self, types):
+        "Tyeps is an array of strings. Check if any of those types is present"
+        tmp = None
+        if hasattr(self, 'REQUEST'):
+            # hideWhens need a TemporaryDocument
+            tmp = TemporaryDocument(self.getParentDatabase(), self, self.REQUEST)
+        fields = self.getFormFields(includesubforms=True, doc=tmp, applyhidewhen=True)
         for f in fields:
-            if f.getFieldType() in ["DATETIME", "DATAGRID"]:
+            if f.getFieldType() in types:
                 return True
         return False
 
@@ -685,23 +705,18 @@ class PlominoForm(ATFolder):
     def hasGoogleVisualizationField(self):
         """return true if the form contains at least one GoogleVisualization field
         """
-        fields = self.getFormFields(includesubforms=True, applyhidewhen=True)
-        for f in fields:
-            if f.getFieldType() == "GOOGLEVISUALIZATION":
-                return True
-        return False
+        return self._has_fieldtypes(["GOOGLEVISUALIZATION"])
 
     security.declarePublic('getSubforms')
     def getSubforms(self, doc=None, applyhidewhen=True):
         """return the names of the subforms embedded in the form
         """
         if applyhidewhen:
+            if doc == None and hasattr(self, 'REQUEST'):
+                doc = TemporaryDocument(self.getParentDatabase(), self, self.REQUEST)
             html_content = self.applyHideWhen(doc)
         else:
-            plone_tools = getToolByName(self, 'plone_utils')
-            encoding = plone_tools.getSiteEncoding()
-            html_content = self.getField('FormLayout').getRaw(self).decode(encoding)
-            html_content = html_content.replace('\n', '')
+            html_content = self._get_html_content()
 
         r = re.compile('<span class="plominoSubformClass">([^<]+)</span>')
         return [i.strip() for i in r.findall(html_content)]
@@ -806,13 +821,34 @@ class PlominoForm(ATFolder):
             return self.errors_json(errors=json.dumps({'success': False,'errors':errors}))
         else:
             return self.errors_json(errors=json.dumps({'success': True}))
-        
+
+
+    security.declarePrivate('_get_js_hidden_fields')
+    def _get_js_hidden_fields(self, REQUEST, doc):
+        hidden_fields = []
+        hidewhens = json.loads(self.getHidewhenAsJSON(REQUEST))
+        html_content = self._get_html_content()
+        for hidewhenName, doit in hidewhens.items():
+            if not doit: # Only consider True hidewhens
+                continue
+            start = '<span class="plominoHidewhenClass">start:'+hidewhenName+'</span>'
+            end = '<span class="plominoHidewhenClass">end:'+hidewhenName+'</span>'
+            for hiddensection in re.findall(start + '(.*?)' + end, html_content):
+                hidden_fields += re.findall(
+                    '<span class="plominoFieldClass">([^<]+)</span>', hiddensection )
+        for subformname in self.getSubforms(doc):
+            subform = self.getParentDatabase().getForm(subformname)
+            hidden_fields += subform._get_js_hidden_fields(REQUEST, doc)
+        return hidden_fields
+
     security.declarePublic('validateInputs')
     def validateInputs(self, REQUEST, doc=None):
         """
         """
         errors=[]
         fields = self.getFormFields(includesubforms=True, doc=doc, applyhidewhen=True)
+        hidden_fields = self._get_js_hidden_fields(REQUEST, doc)
+        fields = [field for field in fields if field.getId() not in hidden_fields]
         for f in fields:
             fieldname = f.id
             fieldtype = f.getFieldType()
@@ -875,7 +911,7 @@ class PlominoForm(ATFolder):
                          SelectionFormula=formula)
         view_obj = getattr(db, view_id)
         view_obj.at_post_create_script()
-        
+
         fields = self.getFormFields(includesubforms=True)
         acceptable_types = ["TEXT", "NUMBER", "NAME", "SELECTION", "DATETIME"]
         fields = [f for f in fields if f.getFieldMode()=="EDITABLE" and f.FieldType in acceptable_types]
@@ -894,10 +930,10 @@ class PlominoForm(ATFolder):
                                   ActionType="OPENFORM",
                                   ActionDisplay="BUTTON",
                                   Content=self.id)
-        
+
         if REQUEST:
             REQUEST.RESPONSE.redirect(view_obj.absolute_url_path())
-        
+
     security.declarePublic('getPosition')
     def getPosition(self):
         """Return the form position in the database
@@ -919,7 +955,7 @@ class PlominoForm(ATFolder):
         """
         """
         return False
-    
+
     security.declarePublic('isEditMode')
     def isEditMode(self):
         """
@@ -940,7 +976,7 @@ class PlominoForm(ATFolder):
         if REQUEST:
             REQUEST.RESPONSE.setHeader('content-type', 'application/json; charset=utf-8')
             item = REQUEST.get('item', item)
-        
+
         result = None
         if not item:
             fields = self.getFormFields()
@@ -954,8 +990,8 @@ class PlominoForm(ATFolder):
             if field:
                 adapt = field.getSettings()
                 result = adapt.getFieldValue(self, None, False, False, REQUEST)
-        
-        return json.dumps(result) 
+
+        return json.dumps(result)
 
     def _getDatabaseViews(self):
         db = self.getParentDatabase()
@@ -963,5 +999,3 @@ class PlominoForm(ATFolder):
         return [''] +  [v.id for v in views]
 
 registerType(PlominoForm, PROJECTNAME)
-
-
